'use strict';

var _interopRequireWildcard = function (obj) { return obj && obj.__esModule ? obj : { 'default': obj }; };

Object.defineProperty(exports, '__esModule', {
  value: true
});

require('html5-history-api');

require('native-promise-only');

var _DOMReady = require('./domready');

var _DOMReady2 = _interopRequireWildcard(_DOMReady);

var _Events = require('vertebrae/events');

var _Events2 = _interopRequireWildcard(_Events);

var _page = require('page');

var _page2 = _interopRequireWildcard(_page);

var _qs = require('qs');

var _qs2 = _interopRequireWildcard(_qs);

var _Store = require('./store');

var _Store2 = _interopRequireWildcard(_Store);

var _utils = require('./utils');

var _utils2 = _interopRequireWildcard(_utils);

var _validateOptions = require('./validate-options');

var _validateOptions2 = _interopRequireWildcard(_validateOptions);

var _extend = require('vertebrae/utils');

function defaultRouteHandler(handler, params) {
  return handler.apply(this, params);
}

function createStoreOptions() {
  var options = arguments[0] === undefined ? {} : arguments[0];

  var storeOptions = {};

  if (options.autoClearCaches) {
    storeOptions.autoClearCaches = true;
  }

  if (typeof options.autoToJSON !== 'undefined') {
    storeOptions.autoToJSON = options.autoToJSON;
  }

  if (typeof options.instantResolve !== 'undefined') {
    storeOptions.instantResolve = options.instantResolve;
  }

  if (typeof options.allowedStatusCodes !== 'undefined') {
    storeOptions.allowedStatusCodes = options.allowedStatusCodes;
  }

  return storeOptions;
}

function Client() {
  var options = arguments[0] === undefined ? {} : arguments[0];
  var routeContext = arguments[1] === undefined ? {} : arguments[1];

  _validateOptions2['default'](options);

  // ensure proper initial state for page.js
  _page2['default'].stop();
  _page2['default'].callbacks = [];
  _page2['default'].exits = [];

  var initializeCallback = options.initialize;
  var _options$initStore = options.initStore;
  var initStore = _options$initStore === undefined ? true : _options$initStore;
  var initialUrl = options.initialUrl;
  var render = options.render;
  var routes = options.routes;
  var _options$routeHandler = options.routeHandler;
  var routeHandler = _options$routeHandler === undefined ? defaultRouteHandler : _options$routeHandler;
  var storeId = options.storeId;
  var stores = options.stores;

  var clientEvents = _extend.extend({}, _Events2['default']);
  var store = initStore ? new _Store2['default'](stores, createStoreOptions(options)) : null;

  _DOMReady2['default'].then(function () {

    if (initStore) {
      var storeState = document.getElementById(storeId);
      store.bootstrap(storeState.innerHTML);
      storeState.innerHTML = '';
    }

    // register page.js handler for each route
    Object.keys(routes).forEach(function (route) {

      _page2['default'](route, function (ctx) {
        var context = typeof routeContext === 'function' ? routeContext.call({}) : routeContext;

        // return array of params in proper order
        var params = _utils2['default'].extractParams(route, ctx.params);

        // add parsed query string object as last parameter for route handler
        var query = _qs2['default'].parse(ctx.querystring);
        params.push(query);

        Promise.resolve(context).then(function (context) {
          if (initStore) {
            context.store = store;
          }

          return Promise.resolve(routeHandler.call(context, routes[route], params)).then(function (options) {
            return Promise.resolve(render.call(context, document, options, { params: ctx.params, query: query })).then(function (result) {
              clientEvents.trigger('render', route);
              return result;
            });
          })['catch'](function (error) {
            // log error as user hasn't handled it
            console.error('Render error while processing route ' + route + ':, ' + error);
          });
        });
      });
    });

    // initialize page.js route handling
    _page2['default'](initialUrl);

    // invoke initialize callback if it was provided in options
    if (initializeCallback && typeof initializeCallback === 'function') {
      initializeCallback.call(null, {
        router: _page2['default'],
        store: store
      });
    }
  });

  return clientEvents;
};

exports['default'] = Client;
module.exports = exports['default'];