'use strict';

var _interopRequireWildcard = function (obj) { return obj && obj.__esModule ? obj : { 'default': obj }; };

Object.defineProperty(exports, '__esModule', {
  value: true
});

require('native-promise-only');

var _cheerio = require('cheerio');

var _cheerio2 = _interopRequireWildcard(_cheerio);

var _express = require('express');

var _express2 = _interopRequireWildcard(_express);

var _Store = require('./store');

var _Store2 = _interopRequireWildcard(_Store);

var _utils = require('./utils');

var _utils2 = _interopRequireWildcard(_utils);

var _serverUtils = require('./server-utils');

var _serverUtils2 = _interopRequireWildcard(_serverUtils);

var _validateOptions = require('./validate-options');

var _validateOptions2 = _interopRequireWildcard(_validateOptions);

function defaultRouteHandler(handler, params) {
  return handler.apply(this, params);
}

function Server() {
  var options = arguments[0] === undefined ? {} : arguments[0];
  var routeContext = arguments[1] === undefined ? {} : arguments[1];

  _validateOptions2['default'](options);

  var _options$app = options.app;
  var appSettings = _options$app === undefined ? [] : _options$app;
  var _options$entries = options.entries;
  var entries = _options$entries === undefined ? {
    path: null,
    routes: {}
  } : _options$entries;
  var _options$middleware = options.middleware;
  var middleware = _options$middleware === undefined ? [] : _options$middleware;
  var render = options.render;
  var routes = options.routes;
  var _options$routeHandler = options.routeHandler;
  var routeHandler = _options$routeHandler === undefined ? defaultRouteHandler : _options$routeHandler;
  var staticFiles = options.staticFiles;
  var storeId = options.storeId;
  var stores = options.stores;
  var _options$autoToJSON = options.autoToJSON;
  var autoToJSON = _options$autoToJSON === undefined ? true : _options$autoToJSON;
  var _options$initStore = options.initStore;
  var initStore = _options$initStore === undefined ? true : _options$initStore;
  var _options$allowedStatusCodes = options.allowedStatusCodes;
  var allowedStatusCodes = _options$allowedStatusCodes === undefined ? null : _options$allowedStatusCodes;

  if (!staticFiles || typeof staticFiles !== 'string') {
    throw new Error('You must define staticFiles path for index.html');
  }

  // preload all entry files on startup
  var entryFiles = _serverUtils2['default'].loadEntries(entries, staticFiles);
  var app = _express2['default']();

  // useful for delaying the static middleware injection
  app.useStatic = function () {
    app.use(_express2['default']['static'](staticFiles));
  };

  appSettings.forEach(function (key) {
    app.set(key, appSettings[key]);
  });

  middleware.forEach(function (mw) {
    if (mw.constructor === Array) {
      app.use.apply(app, mw);
    } else {
      app.use(mw);
    }
  });

  Object.keys(routes).forEach(function (route) {

    app.get(route, function (req, res) {
      var context = typeof routeContext === 'function' ? routeContext.call({}, req) : routeContext;

      // return array of params in proper order
      var params = _utils2['default'].extractParams(route, req.params);

      // add parsed query string object as last parameter for route handler
      params.push(req.query);

      Promise.resolve(context).then(function (context) {
        if (initStore) {
          var _options = {
            autoToJSON: autoToJSON,
            allowedStatusCodes: allowedStatusCodes
          };

          if (req.headers.cookie) {
            _options.cookie = req.headers.cookie;
          }

          context.store = new _Store2['default'](stores, _options);
        }

        return Promise.resolve(routeHandler.call(context, routes[route], params)).then(function (options) {
          var document = _cheerio2['default'].load(_serverUtils2['default'].entryHTML(entryFiles, req));

          // store state snapshot to HTML document
          if (context.store) {
            document('#' + storeId).text(context.store.snapshot());
          }

          return Promise.resolve(render.call(context, document, options, {
            params: req.params,
            query: req.query
          })).then(function (response) {
            return res.send(response);
          });
        })['catch'](function (error) {
          if (app.routeError && typeof app.routeError === 'function') {
            app.routeError(error);
          }
          if (error.status && error.data) {
            return res.send('Error ' + error.status + ': ' + error.data);
          } else {
            return res.send('Error: ' + error.stack);
          }
        });
      });
    });
  });

  return app;
};

exports['default'] = Server;
module.exports = exports['default'];